/*
 * Copyright (c) 2012 Flavio Ribeiro <fr@lps.usp.br>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

import QtQuick 1.1
import com.nokia.meego 1.0

Page {
    id: helpPage
    tools: infopageTools
    orientationLock: PageOrientation.LockPortrait

    Flickable {
        anchors.fill: parent
        clip: true
        contentWidth: help_text.paintedWidth
        contentHeight: help_text.paintedHeight
        flickableDirection: Flickable.VerticalFlick
        boundsBehavior: Flickable.StopAtBounds
        Text  {
            id: help_text
            font.pixelSize: 20
            color: "white"
            width: helpPage.width
            wrapMode: Text.WordWrap
            horizontalAlignment: Text.AlignLeft
            text:
'<html>

<style type="text/css">
    td { padding-left: 10px; padding-right: 10px; }
    b.math { color: #99FFFF; white-space: nowrap; }
    b.func { color: #99FF99; white-space: nowrap; }
</style>

<center><h1>RPN9 1.0.0</h1></center>

<p>RPN9 is a full-featured RPN calculator. This help file explains how to use it, and should be useful to
both novice and experienced users.</p>

<!-- About -->
<h2>About RPN</h2>
<p>Reverse Polish Notation (RPN) is a mathematical notation where operators follow their operands.
For instance, the algebraic expression <b class="math">2 + 3</b> is written in RPN as
<b class="math">2 3 +</b>.</p>

<p>An RPN expression is always evaluated by applying operators in left-to-right order. Thus,
<b class="math">2 3 + 1 -</b> reduces to <b class="math">5 1 -</b>, which reduces to <b class="math">4</b>.</p>

<p>RPN is convenient because it never requires parentheses. For example, the algebraic expression
<b class="math">5 - (1 + 2)</b> can be written in RPN as <b class="math">5 1 2 + -</b>.</p>

<p><b class="func">ENTER</b> is used to input values which are used for computation.
When pressed, it commits the most recently typed value to a stack. When a function key is pressed, its arguments are consumed from
the stack and replaced with the function\'s result.</p>

<p>Therefore, <b class="math">2 3 + 1 -</b> is computed with</p>

    <p style="margin-left: 50px;"><b class="func">2 ENTER 3 ENTER + 1 ENTER -</b></p>

<p>The <b class="func">ENTER</b> preceding an operator is optional, such that the previous expression is equivalent to</p>

    <p style="margin-left: 50px;"><b class="func">2 ENTER 3 + 1 -</b></p>

<p>The stack is shown on RPN9\'s display. By flicking the display, all values of the stack can be read.
We refer to the 4 most recent values of the stack as <b>x</b>, <b>y</b>, <b>z</b> and <b>w</b>, with
<b>x</b> being the most recent one.</p>

<!-- Stack -->
<h2>Using the stack</h2>
<p>After entering a value, press <b class="func">ENTER</b> to commit it to the stack. Repeated presses will commit additional copies.
The backspace key <b class="func">⌫</b> can be used do delete individual digits while the number is being typed, or to remove a
value from the stack after it has been entered.</p>

<p><b class="func">x⇄y</b> exchanges <b>x</b> and <b>y</b>. <b class="func">x⇄</b> exchanges <b>x</b> with the value
from an A-Z memory slot. Likewise, <b class="func">y⇄</b> exchanges <b>y</b> with the value from a memory slot.
<b class="func">R↓</b> and <b class="func">R↑</b> rotate the stack down and up, respectively.</p>

<p><b class="func">UNDO</b> reverts the effects of the last operation. It can be repeated as needed.
<b class="func">REDO</b> reverts the <b class="func">UNDO</b>.</p>

<p><b class="func">LAST x</b> commits the value of <b>x</b> before the last operation, without undoing it.
While held down, <b class="func">SHOW</b> renders the stack in smaller font and with additional significant digits.</p>

<!-- scientific functions -->
<h2>Scientific functions</h2>

<table>
  <tr>
    <td><b class="func">+</b>, <b class="func">-</b>, <b class="func">×</b>, <b class="func">÷</b></td>
    <td>arithmetic operators</td>
  </tr>
  <tr>
    <td><b class="func">+/-</b></td>
    <td>changes the sign of a number</td>
  </tr>
  <tr>
    <td><b class="func">r</b></td>
    <td>remainder</td>
  </tr>
  <tr>
    <td><b class="func">√x</b>, <b class="func">x<sup>2</sup></b></td>
    <td>square root and square</td>
  </tr>
  <tr>
    <td><b class="func"><sup>y</sup>√x</b>, <b class="func">y<sup>x</sup></b></td>
    <td><b>y</b><sup>th</sup> root and <b>y</b><sup>th</sup> power of <b>x</b></b></td>
  </tr>
  <tr>
    <td><b class="func">LN</b>, <b class="func">LOG</b></td>
    <td>natural and base-10 logarithms</td>
  </tr>
  <tr>
    <td><b class="func">e<sup>x</sup></b>, <b class="func">10<sup>x</sup></b></td>
    <td>natural and base-10 exponentials</td>
  </tr>
  <tr>
    <td><b class="func">1/x</b>, <b class="func">x!</b></td>
    <td>inverse and factorial</td>
  </tr>
  <tr>
    <td><b class="func">SIN</b>, <b class="func">COS</b>, <b class="func">TAN</b></td>
    <td>sine, cosine and tangent, in degrees or radians (selectable using <b class="func">MODE</b>)</td>
  </tr>
  <tr>
    <td><b class="func">CPLX</b></td>
    <td>prefix for complex operators</td>
  </tr>
  <tr>
    <td><b class="func">HYP</b></td>
    <td>prefix for hyperbolic trigonometry</td>
  </tr>
  <tr>
    <td><b class="func">%</b></td>
    <td><b>x</b>% of <b>y</b></td>
  </tr>
  <tr>
    <td><b class="func">%Δ</b></td>
    <td>% change from <b>y</b> to <b>x</b></td>
  </tr>
  <tr>
    <td><b class="func">C<sub>n,k</sub></b>, <b class="func">P<sub>n,k</sub></b></td>
    <td>combinations and permutations of n elements, taken k elements at a time</td>
  </tr>
  <tr>
    <td><b class="func">RND</b></td>
    <td>rounds to the nearest integer</td>
  </tr>
  <tr>
    <td><b class="func">RAND</b></td>
    <td>pseudo-random number in [0,1)</td>
  </tr>
  <tr>
    <td><b class="func">SRAND</b></td>
    <td>pseudo-random number generator seed</td>
  </tr>
  <tr>
    <td><b class="func">STO</b>, <b class="func">RCL</b></td>
    <td>stores and recalls values from the A-Z memory slots</td>
  </tr>
</table>

<!-- complex functions -->
<h2>Complex numbers</h2>
<p>Several operations with complex numbers are supported with the <b class="func">CPLX</b> prefix. The major distiction
is that complex operands have real and imaginary parts, which must be entered separately. A complex number
<b class="math">x + iy</b> is entered by first typing the imaginary part <b class="math">y</b>, pressing
<b class="func">ENTER</b> and then typing the real part <b class="math">x</b>.</p>

<p>Thus, complex sums, subtractions, multiplications and divisions require entering 4 values each, corresponding to the real and
imaginary parts of the operands. They leave 2 values on the stack, corresponding to the real and imaginary parts of the result.</p>

<p><b class="func">CPLX</b> can be applied to arithmetic operators, trigonometric functions, <b class="func">x<sup>2</sup></b>,
<b class="func">√x</b>, <b class="func">y<sup>x</sup></b>, <b class="func"><sup>y</sup>√x</b>,
<b class="func">e<sup>x</sup></b>, <b class="func">LN</b> and <b class="func">1/x</b>.</p>

<!-- statistical functions -->
<h2>Statistical operations</h2>
<p>RPN9 can be used for estimating:</p>

<ul>
  <li>arithmetic means</li>
  <li>weighted means</li>
  <li>standard deviations</li>
  <li>Pearson correlation coefficients</li>
  <li>linear regressions.</li>
</ul>

<p>To estimate these values, RPN9 expects a sequence of <b class="math">N</b> pairs
<b class="math">(x<sub>i</sub>,y<sub>i</sub>)</b>, for <b class="math">0 &le; i &lt; N</b>. These should be entered one at a time, followed by
<b class="func">Σ+</b>. If one wishes to estimate a quantity which only depends on a single variable (for instance, a standard
deviation), the <b class="math">y<sub>i</sub></b> values are irrelevant.</p>

<p>After entering the entire sequence, the following functions are available:</p>

<table>
  <tr>
    <td><b class="func">SUMS</b></td>
    <td>used to obtain <b class="math">N</b>, <b class="math">Σx<sub>i</sub></b>, <b class="math">Σy<sub>i</sub></b>,
        <b class="math">Σx<sub>i</sub>y<sub>i</sub></b>, <b class="math">Σx<sub>i</sub><sup>2</sup></b> and
        <b class="math">Σy<sub>i</sub><sup>2</sup></b></td>
  </tr>
  <tr>
    <td><b class="func">σ<sub>x</sub>, σ<sub>y</sub></b></td>
    <td>used to obtain <b class="math">σ<sub>x</sub></b> and <b class="math">σ<sub>y</sub></b> (standard deviations of
        <b class="math">x</b> and <b class="math">y</b>)</td>
  </tr>
  <tr>
    <td><b class="func"><font style="text-decoration: overline;">x</font>,<font style="text-decoration: overline;">y</font></b></td>
    <td>used to obtain the means of <b class="math">x</b> and <b class="math">y</b></td>
  </tr>
  <tr>
    <td><b class="func">L.R.</b></td>
    <td>least-squares linear regression</td>
  </tr>
</table>

<!-- rational approximations -->
<h2>Rational approximations</h2>
<p>Given a real number <b class="math">x</b>, RPN9 can be used to produce a rational approximation
<b class="math">p/q</b> such that <b class="math">p/q ≈ x</b>, under the constraint that
<b class="math">q &lt; q<sub>max</sub></b> for a configurable <b class="math">q<sub>max</sub></b>.</p>

<p>To configure <b class="math">q<sub>max</sub></b>, simply enter it and press <b class="func">/q</b>.
The default value is <b class="math">q<sub>max</sub> = 4096</b>. To obtain a rational approximation, simply press
and hold <b class="func">FDISP</b>.</p>

<!-- logical operators -->
<h2>Logical operators</h2>
<p>RPN9 can be used to compute logical <b class="func">AND</b>, <b class="func">NAND</b>, <b class="func">OR</b>,
<b class="func">NOR</b>, <b class="func">XOR</b>, <b class="func">XNOR</b>, <b class="func">NOT</b>,
two\'s complement (<b class="func">2CMP</b>), right shifts (<b class="func">SHR</b>) and left shifts (<b class="func">SHL</b>).
For these operators, it may be appropriate to switch to hexadecimal, octal or binary display, using <b class="func">BASE</b>.</p>

<p>The <b class="func">b⇄</b> key is convenient for quickly changing back to the last base. Please note that base conversions and
logical operators are performed with 32-bit registers.</p>

<!-- contact -->
<h2>Contact information</h2>
<p>Feel free to submit comments and bug reports to Flavio&nbsp;Ribeiro&nbsp;&lt;fr@lps.usp.br&gt;.</p>
<br>

</html>';
        }
    }

    ToolBarLayout {
        id: infopageTools
        visible: true
        ToolIcon {
            platformIconId: "toolbar-back"
            onClicked: appWindow.pageStack.pop()
        }
    }
}
