/*
 * Copyright (c) 2012 Flavio Ribeiro <fr@lps.usp.br>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

import QtQuick 1.1
import QtMobility.feedback 1.1
import com.nokia.meego 1.0
import "calculator.js" as C
import "storage.js" as Storage

Page {
    id: mainPage
    orientationLock: PageOrientation.LockPortrait

    property real defButtonSpaceH: 5
    property real defButtonSpaceV: 20
    property real funButtonWidth: parent.width/6 - defButtonSpaceH
    property real numButtonWidth: parent.width/5 - defButtonSpaceH
    property real defButtonHeight: 69
    property int  funFontSize: 28
    property int  numFontSize: 36
    property int  abcFontSize: 16
    property int  altFontSize: 14

    Component.onCompleted: {
        Storage.initDatabase();
        C.initialize();
    }

    FileEffect {
        id: vibra_press
        loaded: false
        source: "file:///usr/share/themes/base/meegotouch/feedbacks/priority2_static_press/vibra.ivt"
    }

    FileEffect {
        id: vibra_release
        loaded: false
        source: "file:///usr/share/themes/base/meegotouch/feedbacks/priority2_static_release/vibra.ivt"
    }

    ButtonStyle {
        id: bs_function
        fontPixelSize: funFontSize
        buttonWidth: funButtonWidth
        buttonHeight: defButtonHeight
        pressedTextColor: "white"
    }

    ButtonStyle {
        id: bs_enter
        fontPixelSize: funFontSize
        buttonWidth: 2*funButtonWidth + defButtonSpaceH
        buttonHeight: defButtonHeight
        pressedTextColor: "white"
    }

    ButtonStyle {
        id: bs_arithmetic
        fontPixelSize: numFontSize
        buttonWidth: numButtonWidth
        buttonHeight: defButtonHeight
        pressedTextColor: "white"
    }

    ButtonStyle {
        id: bs_alt_right
        fontPixelSize: numFontSize
        buttonWidth: numButtonWidth
        buttonHeight: defButtonHeight
        pressedTextColor: "white"

        background: "qrc:///images/color6-meegotouch-button-inverted-background-pressed.png"
        pressedBackground: "qrc:///images/color6-meegotouch-button-inverted-background-selected.png"
        checkedBackground: "qrc:///images/color6-meegotouch-button-inverted-background-selected.png"
        checkedDisabledBackground: "qrc:///images/color6-meegotouch-button-inverted-background-disabled-selected.png"
    }

    ButtonStyle {
        id: bs_alt_left
        fontPixelSize: numFontSize
        buttonWidth: numButtonWidth
        buttonHeight: defButtonHeight
        pressedTextColor: "white"

        background: "qrc:///images/color18-meegotouch-button-inverted-background-pressed.png"
        pressedBackground: "qrc:///images/color18-meegotouch-button-inverted-background-selected.png"
        checkedBackground: "qrc:///images/color18-meegotouch-button-inverted-background-selected.png"
        checkedDisabledBackground: "qrc:///images/color18-meegotouch-button-inverted-background-disabled-selected.png"
    }

    ButtonStyle {
        id: bs_on
        fontPixelSize: numFontSize
        buttonWidth: numButtonWidth
        buttonHeight: defButtonHeight
        pressedTextColor: "white"
    }

    ButtonStyle {
        id: bs_help
        buttonWidth: 64
        buttonHeight: 60
        background: ""
    }

    Rectangle {
        id: display_area
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        height: 180
        radius: 16
        color: "#181818"

        Button {
            id: button_help;
            z: 1
            opacity: 0.5
            anchors.top: parent.top
            anchors.left: parent.left
            platformStyle: bs_help
            iconSource: "image://theme/icon-m-toolbar-view-menu-white"
            onClicked: appWindow.pageStack.push(helpPage);
        }

        Text {
            id: error_area
            anchors.fill: parent
            verticalAlignment: Text.AlignVCenter
            horizontalAlignment: Text.AlignHCenter
            text: "!"
            color: "red"
            font.pixelSize: 80
            visible: false
            z: 1
        }

        Timer {
            id: error_area_timer;
            interval: 500; running: false; repeat: false;
            onTriggered: error_area.visible = false;
        }

        Flickable {
            id: display_flickable
            anchors.bottom: parent.bottom
            anchors.right: parent.right
            anchors.bottomMargin: 20
            anchors.rightMargin: 20
            // width/height workaround, because Flickable left-aligns children who are smaller than it
            width: display.paintedWidth < (parent.width - 40) ? display.paintedWidth : (parent.width - 40)
            height: display.paintedHeight < (parent.height - 40) ? display.paintedHeight : (parent.height - 40)
            contentWidth: display.paintedWidth
            contentHeight: display.paintedHeight
            flickableDirection: Flickable.HorizontalAndVerticalFlick
            boundsBehavior: Flickable.StopAtBounds
            clip: true

            Text {
                id: display
                horizontalAlignment: TextEdit.AlignRight
                verticalAlignment: TextEdit.AlignBottom
                color: "white"
                font.bold: true
                font.family: "DejaVu Sans Mono"
                text: ""
            }
        }

        Text {
            id: alt
            anchors.bottom: parent.bottom
            anchors.horizontalCenter: parent.left
            anchors.horizontalCenterOffset: x_coord(0,0) + funButtonWidth/2
            color: "white"
            font.pixelSize: 16
            font.bold: true
            text: ""
        }

        Text {
            id: mode
            anchors.bottom: parent.bottom
            anchors.horizontalCenter: parent.left
            anchors.horizontalCenterOffset: x_coord(0,1) + funButtonWidth/2
            color: "white"
            font.pixelSize: 16
            font.bold: true
            text: ""
        }

        Text {
            id: disp
            anchors.bottom: parent.bottom
            anchors.horizontalCenter: parent.left
            anchors.horizontalCenterOffset: x_coord(0,2) + funButtonWidth/2
            color: "white"
            font.pixelSize: 16
            font.bold: true
            text: ""
        }

        Text {
            id: base
            anchors.bottom: parent.bottom
            anchors.horizontalCenter: parent.left
            anchors.horizontalCenterOffset: x_coord(0,3) + funButtonWidth/2
            color: "white"
            font.pixelSize: 16
            font.bold: true
            text: ""
        }
    }

    MenuStyle {
        id: ms

        // bug fix for PR 1.1
        property url background: "image://theme/meegotouch-menu-background" + __invertedString
    }

    Menu {
        id: mode_menu
        platformStyle: ms
        MenuLayout {
            MenuItem { text: "Angle unit: degrees"  ; onClicked: C.set_option('angle_unit',        C.AngleOptions.Degrees,  true) }
            MenuItem { text: "Angle unit: radians"  ; onClicked: C.set_option('angle_unit',        C.AngleOptions.Radians,  true) }
            MenuItem { text: "Decimal separator: ." ; onClicked: C.set_option('decimal_separator', C.DecimalOptions.Period, true) }
            MenuItem { text: "Decimal separator: ," ; onClicked: C.set_option('decimal_separator', C.DecimalOptions.Comma,  true) }
        }
    }

    Menu {
        id: disp_menu
        platformStyle: ms
        MenuLayout {
            MenuItem { text: "Automatic";   onClicked: { C.set_option('display_mode', C.DisplayOptions.Automatic, true) } }
            MenuItem { text: "Fixed";       onClicked: { C.try_state_change(C.CalculatorState.DispSetup, 0); C.option['display_mode_new'] = C.DisplayOptions.Fixed       ; C.update_display() } }
            MenuItem { text: "Scientific";  onClicked: { C.try_state_change(C.CalculatorState.DispSetup, 0); C.option['display_mode_new'] = C.DisplayOptions.Scientific  ; C.update_display() } }
            MenuItem { text: "Engineering"; onClicked: { C.try_state_change(C.CalculatorState.DispSetup, 0); C.option['display_mode_new'] = C.DisplayOptions.Engineering ; C.update_display() } }
        }
    }

    Menu {
        id: base_menu
        platformStyle: ms
        MenuLayout {
            MenuItem { text: "Decimal (full precision)"        ; onClicked: C.set_option('integer_base', C.BaseOptions.Decimal,     true) }
            MenuItem { text: "Hexadecimal (integer precision)" ; onClicked: C.set_option('integer_base', C.BaseOptions.Hexadecimal, true) }
            MenuItem { text: "Octal (integer precision)"       ; onClicked: C.set_option('integer_base', C.BaseOptions.Octal,       true) }
            MenuItem { text: "Binary (integer precision)"      ; onClicked: C.set_option('integer_base', C.BaseOptions.Binary,      true) }
        }
    }

    Menu {
        id: clear_menu
        platformStyle: ms
        MenuLayout {
            MenuItem { text: "Stack and undo memory" ; onClicked: { C.memory_clear_stack(); C.update_display(); } }
            MenuItem { text: "A-Z memory"            ; onClicked: { C.memory_clear_AZ();   } }
            MenuItem { text: "Σ variables"           ; onClicked: { C.memory_clear_sums(); } }
        }
    }

    Menu {
        id: sums_menu
        platformStyle: ms
        MenuLayout {
            MenuItem { text: "n (number of x,y pairs)"           ; onClicked: { C.op(1,0,'stats_n');  C.update_display(); } }
            MenuItem { text: "Σx (sum of x values)"              ; onClicked: { C.op(1,0,'stats_x');  C.update_display(); } }
            MenuItem { text: "Σy (sum of y values)"              ; onClicked: { C.op(1,0,'stats_y');  C.update_display(); } }
            MenuItem { text: "Σxy (sum of xy products)"          ; onClicked: { C.op(1,0,'stats_xy'); C.update_display(); } }
            MenuItem { text: "Σx<sup>2</sup> (sum of x squares)" ; onClicked: { C.op(1,0,'stats_x2'); C.update_display(); } }
            MenuItem { text: "Σy<sup>2</sup> (sum of y squares)" ; onClicked: { C.op(1,0,'stats_y2'); C.update_display(); } }
        }
    }

    Menu {
        id: mean_menu
        platformStyle: ms
        MenuLayout {
            MenuItem { text: "<font style='text-decoration: overline;'>x</font> (mean of x)"           ; onClicked: { C.op(1,0,'stats_mx'); C.update_display(); } }
            MenuItem { text: "<font style='text-decoration: overline;'>y</font> (mean of y)"           ; onClicked: { C.op(1,0,'stats_my'); C.update_display(); } }
            MenuItem { text: "<font style='text-decoration: overline;'>x</font>w (weighted mean of x)" ; onClicked: { C.op(1,0,'stats_wy'); C.update_display(); } }
        }
    }

    Menu {
        id: stddev_menu
        platformStyle: ms
        MenuLayout {
            MenuItem { text: "σ<sub>x</sub> (sample std. dev. of x)" ; onClicked: { C.op(1,0,'stats_sx'); C.update_display(); } }
            MenuItem { text: "σ<sub>y</sub> (sample std. dev. of y)" ; onClicked: { C.op(1,0,'stats_sy'); C.update_display(); } }
        }
    }

    Menu {
        id: lr_menu
        platformStyle: ms
        MenuLayout {
            MenuItem { text: "x estimate from y"       ; onClicked: { C.op(1,1,'stats_xhat'); C.update_display(); } }
            MenuItem { text: "y estimate from x"       ; onClicked: { C.op(1,1,'stats_yhat'); C.update_display(); } }
            MenuItem { text: "correlation coefficient" ; onClicked: { C.op(1,0,'stats_r');    C.update_display(); } }
            MenuItem { text: "slope"                   ; onClicked: { C.op(1,0,'stats_m');    C.update_display(); } }
            MenuItem { text: "y intercept"             ; onClicked: { C.op(1,0,'stats_b');    C.update_display(); } }
        }
    }

    function x_coord(row, col) {
        if (row <= 2)
            return defButtonSpaceH/2 + col * (funButtonWidth + defButtonSpaceH);
        else
            return defButtonSpaceH/2 + col * (numButtonWidth + defButtonSpaceH);
    }

    function y_coord(row, col) {
        return 30 + row * (defButtonHeight + defButtonSpaceV);
    }

    Item {
        anchors.left: parent.left;
        anchors.top: display_area.bottom;

        CalculatorButton { row: 0; col: 0; onPressedChanged: C.key_pressed(pressed,row,col); id: button_sqrt; platformStyle: bs_function; text: "√x" }
        CalculatorButton { row: 0; col: 1; onPressedChanged: C.key_pressed(pressed,row,col); id: button_exp;  platformStyle: bs_function; text: "e<font size='4'><sup>x</sup></font>" }
        CalculatorButton { row: 0; col: 2; onPressedChanged: C.key_pressed(pressed,row,col); id: button_ln;   platformStyle: bs_function; text: "LN" }
        CalculatorButton { row: 0; col: 3; onPressedChanged: C.key_pressed(pressed,row,col); id: button_pow;  platformStyle: bs_function; text: "y<font size='4'><sup>x</sup></font>" }
        CalculatorButton { row: 0; col: 4; onPressedChanged: C.key_pressed(pressed,row,col); id: button_inv;  platformStyle: bs_function; text: "1/x" }
        CalculatorButton { row: 0; col: 5; onPressedChanged: C.key_pressed(pressed,row,col); id: button_sum;  platformStyle: bs_function; text: "Σ+" }

        AltText { key: button_sqrt; text_l: "x<font size='4'><sup>2</sup></font>"; text_r: "Cn,k" }
        AltText { key: button_exp;  text_l: "10<font size='4'><sup>x</sup></font>"; text_r: "Pn,k" }
        AltText { key: button_ln;   text_l: "LOG"; text_r: "L.R." }
        AltText { key: button_pow;  text_l: "<font size='4'><sup>x</sup></font>√y"; text_r: "<font style='text-decoration: overline;'>x</font>,<font style='text-decoration: overline;'>y</font>" }
        AltText { key: button_inv;  text_l: "x!"; text_r: "σ<sub>x</sub>, σ<sub>y</sub>" }
        AltText { key: button_sum;  text_l: "Σ-"; text_r: "SUMS" }

        AbcText { key: button_sqrt; text: "A" }
        AbcText { key: button_exp;  text: "B" }
        AbcText { key: button_ln;   text: "C" }
        AbcText { key: button_pow;  text: "D" }
        AbcText { key: button_inv;  text: "E" }
        AbcText { key: button_sum;  text: "F" }

        CalculatorButton { row: 1; col: 0; onPressedChanged: C.key_pressed(pressed,row,col); id: button_sto; platformStyle: bs_function; text: "STO" }
        CalculatorButton { row: 1; col: 1; onPressedChanged: C.key_pressed(pressed,row,col); id: button_rcl; platformStyle: bs_function; text: "RCL" }
        CalculatorButton { row: 1; col: 2; onPressedChanged: C.key_pressed(pressed,row,col); id: button_rot; platformStyle: bs_function; text: "R↓" }
        CalculatorButton { row: 1; col: 3; onPressedChanged: C.key_pressed(pressed,row,col); id: button_sin; platformStyle: bs_function; text: "SIN" }
        CalculatorButton { row: 1; col: 4; onPressedChanged: C.key_pressed(pressed,row,col); id: button_cos; platformStyle: bs_function; text: "COS" }
        CalculatorButton { row: 1; col: 5; onPressedChanged: C.key_pressed(pressed,row,col); id: button_tan; platformStyle: bs_function; text: "TAN" }

        AltText { key: button_sto; text_l: "CPLX"; text_r: "HYP" }
        AltText { key: button_rcl; text_l: "RAND"; text_r: "SEED" }
        AltText { key: button_rot; text_l: "RND";  text_r: "R↑" }
        AltText { key: button_sin; text_l: "ASIN"; text_r: "<font size='4'>π</font>" }
        AltText { key: button_cos; text_l: "ACOS"; text_r: "%" }
        AltText { key: button_tan; text_l: "ATAN"; text_r: "%Δ" }

        AbcText { key: button_sto; text: "G" }
        AbcText { key: button_rcl; text: "H" }
        AbcText { key: button_rot; text: "I" }
        AbcText { key: button_sin; text: "J" }
        AbcText { key: button_cos; text: "K" }
        AbcText { key: button_tan; text: "L" }

        CalculatorButton { row: 2; col: 0; onPressedChanged: C.key_pressed(pressed,row,col); id: button_enter; platformStyle: bs_enter;    text: "ENTER" }
        CalculatorButton { row: 2; col: 2; onPressedChanged: C.key_pressed(pressed,row,col); id: button_swap;  platformStyle: bs_function; text: "x<font size='2'>⇄</font>y" }
        CalculatorButton { row: 2; col: 3; onPressedChanged: C.key_pressed(pressed,row,col); id: button_pm;    platformStyle: bs_function; text: "+/-" }
        CalculatorButton { row: 2; col: 4; onPressedChanged: C.key_pressed(pressed,row,col); id: button_e;     platformStyle: bs_function; text: "E" }
        CalculatorButton { row: 2; col: 5; onPressedChanged: C.key_pressed(pressed,row,col); id: button_back;  platformStyle: bs_function; iconSource: "image://theme/icon-m-common-backspace-selected" }

        AltText { key: button_enter; text_l: "LAST <i>x</i>"; text_c: "&nbsp;&nbsp;&nbsp;&nbsp;"; text_r:"SHOW" }
        AltText { key: button_swap;  text_l: "x⇄"; text_r: "y⇄" }
        AltText { key: button_pm;    text_l: "DISP"; text_r: "MODE&nbsp;&nbsp;" }
        AltText { key: button_e;     text_l: "FRAC"; text_r: "REDO" }
        AltText { key: button_back;  text_l: "&nbsp;CLR"; text_r: "UNDO" }

        AbcText { key: button_enter; text: "M" }
        AbcText { key: button_swap;  text: "N" }
        AbcText { key: button_pm;    text: "O" }
        AbcText { key: button_e;     text: "P" }

        CalculatorButton { row: 3; col: 0; onPressedChanged: C.key_pressed(pressed,row,col); id: button_ip;  platformStyle: bs_arithmetic; text: "<font size='2'>b⇄</font>" }
        CalculatorButton { row: 3; col: 1; onPressedChanged: C.key_pressed(pressed,row,col); id: button_7;   platformStyle: bs_arithmetic; text: "7" }
        CalculatorButton { row: 3; col: 2; onPressedChanged: C.key_pressed(pressed,row,col); id: button_8;   platformStyle: bs_arithmetic; text: "8" }
        CalculatorButton { row: 3; col: 3; onPressedChanged: C.key_pressed(pressed,row,col); id: button_9;   platformStyle: bs_arithmetic; text: "9" }
        CalculatorButton { row: 3; col: 4; onPressedChanged: C.key_pressed(pressed,row,col); id: button_div; platformStyle: bs_arithmetic; text: "<font size='4'>÷</font>" }

        AltText { key: button_ip;  text_l: "BASE";  text_r: "IP" }
        AltText { key: button_7;   text_l: "➜θ,r"; text_r: "➜y,x" }
        AltText { key: button_8;   text_l: "➜HR";  text_r: "➜HMS" }
        AltText { key: button_9;   text_l: "➜deg"; text_r: "➜rad" }
        AltText { key: button_div; text_l: "AND";   text_r: "NAND" }

        AbcText { key: button_7; text: "Q" }
        AbcText { key: button_8; text: "R" }
        AbcText { key: button_9; text: "S" }

        CalculatorButton { row: 4; col: 0; onPressedChanged: C.key_pressed_alt_l(pressed);   id: button_alt_l; platformStyle: bs_alt_left; checkable: true; iconSource: "image://theme/icon-m-toolbar-undo-white-selected" }
        CalculatorButton { row: 4; col: 1; onPressedChanged: C.key_pressed(pressed,row,col); id: button_4;     platformStyle: bs_arithmetic; text: "4" }
        CalculatorButton { row: 4; col: 2; onPressedChanged: C.key_pressed(pressed,row,col); id: button_5;     platformStyle: bs_arithmetic; text: "5" }
        CalculatorButton { row: 4; col: 3; onPressedChanged: C.key_pressed(pressed,row,col); id: button_6;     platformStyle: bs_arithmetic; text: "6" }
        CalculatorButton { row: 4; col: 4; onPressedChanged: C.key_pressed(pressed,row,col); id: button_mul;   platformStyle: bs_arithmetic; text: "<font size='4'>×</font>" }

        AltText { key: button_4;   text_l: "➜m";  text_r: "➜yds" }
        AltText { key: button_5;   text_l: "➜ml"; text_r: "➜oz" }
        AltText { key: button_6;   text_l: "➜km"; text_r: "➜mi" }
        AltText { key: button_mul; text_l: "OR";   text_r: "NOR" }

        AbcText { key: button_4; text: "T" }
        AbcText { key: button_5; text: "U" }
        AbcText { key: button_6; text: "V" }

        CalculatorButton { row: 5; col: 0; onPressedChanged: C.key_pressed_alt_r(pressed);   id: button_alt_r; platformStyle: bs_alt_right; checkable: true; iconSource: "image://theme/icon-m-toolbar-redo-white-selected" }
        CalculatorButton { row: 5; col: 1; onPressedChanged: C.key_pressed(pressed,row,col); id: button_1;     platformStyle: bs_arithmetic; text: "1" }
        CalculatorButton { row: 5; col: 2; onPressedChanged: C.key_pressed(pressed,row,col); id: button_2;     platformStyle: bs_arithmetic; text: "2" }
        CalculatorButton { row: 5; col: 3; onPressedChanged: C.key_pressed(pressed,row,col); id: button_3;     platformStyle: bs_arithmetic; text: "3" }
        CalculatorButton { row: 5; col: 4; onPressedChanged: C.key_pressed(pressed,row,col); id: button_sub;   platformStyle: bs_arithmetic; text: "<font size='4'>-</font>" }

        AltText { key: button_1;   text_l: "➜kg"; text_r: "➜lb" }
        AltText { key: button_2;   text_l: "➜l";  text_r: "➜gal" }
        AltText { key: button_3;   text_l: "➜cm"; text_r: "➜in" }
        AltText { key: button_sub; text_l: "XOR";  text_r: "XNOR" }

        AbcText { key: button_1; text: "W" }
        AbcText { key: button_2; text: "X" }
        AbcText { key: button_3; text: "Y" }

        CalculatorButton { row: 6; col: 0; onPressedChanged: C.key_pressed_c(pressed);       id: button_on;  platformStyle: bs_on;         text: "C" }
        CalculatorButton { row: 6; col: 1; onPressedChanged: C.key_pressed(pressed,row,col); id: button_0;   platformStyle: bs_arithmetic; text: "0" }
        CalculatorButton { row: 6; col: 2; onPressedChanged: C.key_pressed(pressed,row,col); id: button_p;   platformStyle: bs_arithmetic; text: "." }
        CalculatorButton { row: 6; col: 3; onPressedChanged: C.key_pressed(pressed,row,col); id: button_mod; platformStyle: bs_arithmetic; text: "r" }
        CalculatorButton { row: 6; col: 4; onPressedChanged: C.key_pressed(pressed,row,col); id: button_add; platformStyle: bs_arithmetic; text: "<font size='4'>+</font>" }

        AltText { key: button_on;  text_c: "";      text_r: "OFF" }
        AltText { key: button_0;   text_l: "➜°C";  text_r: "➜°F" }
        AltText { key: button_p;   text_l: "FDISP"; text_r: "/q" }
        AltText { key: button_mod; text_l: "SHL";   text_r: "SHR" }
        AltText { key: button_add; text_l: "NOT";   text_r: "2CMP" }

        AbcText { key: button_0; text: "Z" }
    }
}
