/*
 * Copyright (c) 2012 Flavio Ribeiro <fr@lps.usp.br>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

function factorial_int(n) {
    var f = 1;
    for (; n > 0; n--)
        f *= n;
    return f;
}

function factorial_real(z) {
    // Lanczos approximation for the Gamma function, from Numerical Recipes in C
    // http://www.rskey.org/CMS/index.php/the-library/11

    var q = [75122.6331530, 80916.6278952, 36308.2951477, 8687.24529705, 1168.92649479, 83.8676043424, 2.50662827511];
    var num = 0, den = 1;

    z = z + 1;      // z! = Gamma(z+1)
    for (var n = 0; n < q.length; n++)
    {
        num += q[n]*Math.pow(z, n);
        den *= (z + n);
    }

    return num/den * Math.pow(z+5.5, z+0.5) * Math.exp(-(z+5.5));
}

Math.factorial = function (n) {
    return (Math.floor(n) === n) ? factorial_int(n) : factorial_real(n);
}

Math.combinations = function (n,k) {
    var num = 1;
    var den = 1;
    var i;

    for (i = n-k+1; i <= n; i++)
        num *= i;
    for (i = 1; i <= k; i++)
        den *= i;

    return num/den;
}

Math.permutations = function (n,k) {
    var p = 1;
    var i;

    for (i = n-k+1; i <= n; i++)
        p *= i;

    return p;
}

Math.deg2rad = function (x) {
    return x/180*Math.PI;
}

Math.rad2deg = function (x) {
    return x/Math.PI*180;
}

Math.log10 = function (n) {
    return Math.log(n) * Math.LOG10E;
}

Math.sinh = function(x) {
    return (Math.exp(x) - Math.exp(-x)) / 2;
}

Math.asinh = function(x) {
    return Math.log(x + Math.sqrt(x*x + 1));
}

Math.cosh = function(x) {
    return (Math.exp(x) + Math.exp(-x)) / 2;
}

Math.acosh = function(x) {
    return Math.log(x + Math.sqrt(x*x - 1));
}

Math.tanh = function(x) {
    return (Math.exp(x) - Math.exp(-x)) / (Math.exp(x) + Math.exp(-x));
}

Math.atanh = function(x) {
    return 0.5 * Math.log((1+x)/(1-x));
}

// z^w
Math.complex_pow = function(z, w) {
    var t = Math.atan2(z[1], z[0]);
    var r = z[0]*z[0] + z[1]*z[1];
    var xo = Math.pow(r, w[0]/2) * Math.exp(-w[1]*t) * Math.cos(w[0]*t + 0.5*w[1]*Math.log(r));
    var yo = Math.pow(r, w[0]/2) * Math.exp(-w[1]*t) * Math.sin(w[0]*t + 0.5*w[1]*Math.log(r));
    return [xo, yo];
}

// z+w
Math.complex_add = function(z, w) {
    var xo = z[0] + w[0];
    var yo = z[1] + w[1];
    return [xo, yo];
}

// z-w
Math.complex_sub = function(z, w) {
    var xo = z[0] - w[0];
    var yo = z[1] - w[1];
    return [xo, yo];
}

// z*w
Math.complex_mul = function(z, w) {
    var xo = w[0]*z[0] - w[1]*z[1];
    var yo = w[0]*z[1] + w[1]*z[0];
    return [xo, yo];
}

// z/w
Math.complex_div = function(z, w) {
    var xo = (w[0]*z[0] + w[1]*z[1])/(w[0]*w[0] + w[1]*w[1]);
    var yo = (w[0]*z[1] - w[1]*z[0])/(w[0]*w[0] + w[1]*w[1]);
    return [xo, yo];
}

Math.complex_exp = function(z) {
    var xo = Math.exp(z[0])*Math.cos(z[1]);
    var yo = Math.exp(z[0])*Math.sin(z[1]);
    return [xo, yo];
}

Math.complex_ln = function(z) {
    var xo = Math.log(Math.sqrt(z[0]*z[0] + z[1]*z[1]));
    var yo = Math.atan2(z[1],z[0]);
    return [xo, yo];
}

Math.complex_sin = function(z) {
    var xo = Math.sin(z[0])*Math.cosh(z[1]);
    var yo = Math.cos(z[0])*Math.sinh(z[1]);
    return [xo, yo];
}

Math.complex_cos = function(z) {
    var xo =  Math.cos(z[0])*Math.cosh(z[1]);
    var yo = -Math.sin(z[0])*Math.sinh(z[1]);
    return [xo, yo];
}

Math.complex_tan = function(z) {
    var xo = Math.sin(z[0])*Math.cos(z[0])/(Math.pow(Math.cos(z[0]),2) + Math.pow(Math.sinh(z[1]),2));
    var yo = Math.sinh(z[1])*Math.cosh(z[1])/(Math.pow(Math.cos(z[0]),2) + Math.pow(Math.sinh(z[1]),2));
    return [xo, yo];
}

Math.complex_inv = function(z) {
    return Math.complex_div([1,0], z);
}

Math.complex_sqrt = function(z) {
    return Math.complex_pow(z, [0.5,0]);
}

Math.complex_asin = function(z) {
    // -i * ln(i*z + sqrt(1-z^2))
    var s = Math.complex_sqrt(Math.complex_sub([1,0], Math.complex_mul(z,z)));
    var a = Math.complex_add(Math.complex_mul([0,1], z), s);
    return Math.complex_mul([0,-1], Math.complex_ln(a));
}

Math.complex_acos = function(z) {
    // -i * ln(z + sqrt(z^2-1))
    var s = Math.complex_sqrt(Math.complex_sub(Math.complex_mul(z,z), [1,0]));
    var a = Math.complex_add(z, s);
    return Math.complex_mul([0,-1], Math.complex_ln(a));
}

Math.complex_atan = function(z) {
    // (i/2)[ ln(1-i*z) - ln(1+i*z) ]
    var s1 = Math.complex_ln(Math.complex_sub([1,0], Math.complex_mul([0,1],z)));
    var s2 = Math.complex_ln(Math.complex_add([1,0], Math.complex_mul([0,1],z)));
    return Math.complex_mul([0,0.5], Math.complex_sub(s1, s2));
}

// finds rational approximations p/q of x, by computing its continued fraction expansion.
// returns the rational approximation with largest q such that q < q_max.
// (note that even though every continued fraction convergent is a "best" rational approximation,
//  these convergents do not form a complete set)
Math.bestq = function(x, q_max) {
    var y  = x;
    var p0 = 0, q0 = 1;
    var p1 = 1, q1 = 0;
    var p, q;
    var a, r;

    while (true)
    {
        a = Math.floor(y);
        r = y - a;
        y = 1/r;

        p = a*p1 + p0;
        q = a*q1 + q0;

        if (q >= q_max)
            return q1;

        p0 = p1; p1 = p;
        q0 = q1; q1 = q;
    }
}

Math.toInt = function(x) {
    return (x >= 0) ? Math.floor(x) : Math.ceil(x);
}

Math.truncateInt = function(x) {
    return x & 0xFFFFFFFF;
}

// returns the base 10 exponent for scientific notation
Number.prototype.exponent = function(x) {
    var tol = 1e-12;        // to account for the fact that log10 is slightly inaccurate
    return (this == 0) ? 0 : Math.floor(Math.log10(Math.abs(this)) + tol);
}

// returns the number of digits needed to represent the integer part of a number
Number.prototype.intDigits = function(x) {
    var tol = 1e-12;        // to account for the fact that log10 is slightly inaccurate
    return Math.max(Math.floor(Math.log10(Math.abs(this)) + tol),0) + 1;
}

Number.prototype.toStringAuto = function () {
    if (isNaN(this) || !isFinite(this))
        return this.toString();

    var parts, str_fix, str_exp;
    var reqwidth = this.intDigits();
    var N = option['display_width'];

    // if more than N digits are required to represent the integer part,
    // then exponential notation must be used
    if (reqwidth > N)
    {
        parts = this.toExponential(N).split('e');
        parts[0] = parts[0].replace(/\.0*$|(\.\d*[1-9])0*$/, '$1');     // trim trailing zeros
        parts[1] = parts[1].replace(/\+/, '');
        parts[0] = parts[0].substring(0, N - parts[1].length);
        return parts[0] + exp10 + parts[1];
    }
    else
    {
        parts = this.toExponential(N).split('e');
        parts[0] = parts[0].replace(/\.0*$|(\.\d*[1-9])0*$/, '$1');     // trim trailing zeros
        parts[1] = parts[1].replace(/\+/, '');
        parts[0] = parts[0].substring(0, N - parts[1].length);
        str_exp  = parts[0] + exp10 + parts[1];

        str_fix = this.toFixed(N - reqwidth);
        str_fix = str_fix.replace(/\.0*$|(\.\d*[1-9])0*$/, '$1');       // trim trailing zeros

        var error_exp = Math.abs(this - parseFloat(str_exp));
        var error_fix = Math.abs(this - parseFloat(str_fix));

        return (error_fix <= error_exp) ? str_fix : str_exp;
    }
}

Number.prototype.toStringFixed = function (k) {
    if (isNaN(this) || !isFinite(this))
        return this.toString();

    if (this == 0)
        return this.toFixed(k);
    else if (Math.abs(this) >= 1)
    {
        // required width in characters to print the integer and fractional parts
        var reqwidth = this.intDigits() + k;
        return (reqwidth <= option['display_width']) ? this.toFixed(k) : this.toExponential(k);
    }
    else
        return (Math.abs(this) >= 5*Math.pow(10,-k-1)) ? this.toFixed(k) : this.toExponential(k);
}

Number.prototype.toStringSci = function (k) {
    if (isNaN(this) || !isFinite(this))
        return this.toString();

    return this.toExponential(k);
}

Number.prototype.toStringEng = function (k) {
    if (isNaN(this) || !isFinite(this))
        return this.toString();

    var exp      = Math.floor(this.exponent()/3)*3;
    var mantissa = this / Math.pow(10,exp);
    var decimals = Math.max(k - mantissa.intDigits() + 1,0);

    return mantissa.toFixed(decimals) + exp10 + exp.toFixed(0);
}

Number.prototype.toStringUnsigned = function (base) {
    var max_uint = Math.round(Math.pow(2,32));

    var n = Math.toInt(this);
    if (n < 0)
        n += max_uint;

    if (n < 0 || n >= max_uint)
    {
        n = Math.truncateInt(n);
        return '<font color="red">' + n.toString(base).toUpperCase() + '</font>';
    }
    else
        return n.toString(base).toUpperCase();
}

Array.prototype.top = function () {
    return this[this.length-1];
}
